﻿using System;
using System.Collections.Generic;
using System.IO;

namespace ELTE.CSharp.Basics
{
    class Program
    {
        // Beolvasás konzol bementről
        private static void Read(out List<Single> values)
        {
            values = new List<Single>();
            Console.WriteLine("Adatok beolvasása 'q'-ig");
            try // kivételkezelő rész
            {
                String value = Console.ReadLine();
                while (value != "q")
                {
                    values.Add(Convert.ToSingle(value));
                    value = Console.ReadLine();
                }
            }
            catch (FormatException) // kivételek elfogása
            {
                Console.WriteLine("A bemeneten lévő adat nem valós szám!");
            }
        }
        // Beolvasás fájlból
        private static void Read(String fileName, out List<Single> values)
        {
            values = new List<Single>();
            try
            {
                StreamReader reader = new StreamReader(fileName);
                Single value = Convert.ToSingle(reader.ReadLine());
                while (!reader.EndOfStream)
                {
                    values.Add(value);
                    value = Convert.ToSingle(reader.ReadLine());
                }
                reader.Close();
            }
            catch (IOException) // fájllal kapcsolatos kivételek
            {
                Console.WriteLine("Hiba történt az adatok beolvasásakor!");
            }
            catch (FormatException) // formátummal kapcsolatos kivételek
            {
                Console.WriteLine("A bemeneten lévő adat nem valós szám!");
            }
        }
        // Lista kiírása a képernyőre
        private static void Write(List<Single> values)
        {
            for (Int32 i = 0; i < values.Count; i++)
            {
                Console.Write(values[i] + " ");
            }
            Console.WriteLine();
        }
        // Lista elemeinek megfordítása
        private static List<Single> ReverseValues(List<Single> values)
        { 
            Stack<Single> stack = new Stack<Single>();
            List<Single> result = new List<Single>();

            for (Int32 i = 0; i < values.Count; i++)
                stack.Push(values[i]); // berakjuk eze elemeket a verembe
            
            while (stack.Count > 0) // amíg nem üres a verem
                result.Add(stack.Pop()); // kivesszük az elemeket, és betesszük az eredmény listába

            return result;
        }

        static void Main(string[] args)
        {
            List<Single> values = new List<Single>();

            if (args.Length > 0) // ha van parancssori argumentum
                Read(args[0], out values); // azt olvasssuk be
            else // különben megkérdezzük a felhasználót
            {
                Int32 choice;
                do
                {
                    Console.WriteLine("Beolvasás forrása? ");
                    Console.WriteLine("1: Konzol");
                    Console.WriteLine("2: Fájl");
                    choice = Convert.ToInt32(Console.ReadLine());
                    switch (choice) // beolvasás forrásának eldöntése
                    {
                        case 1:
                            Read(out values);
                            break;
                        case 2:
                            Console.WriteLine("Fájlnév: ");
                            String fileName = Console.ReadLine();
                            Read(fileName, out values);
                            break;
                        default:
                            Console.WriteLine("Csak 1 és 2 menüpontok vannak!");
                            break;
                    }
                } while (choice != 1 && choice != 2); // addig írjuk ki, amíg nem ad 1-2 közötti választ
            }

            if (values.Count == 0) // ha nincs beolvasott adatok
                Console.WriteLine("Nincs feldolgozandó adat!");
            else
            {
                Console.WriteLine("Beolvasott adatok:");
                Write(values);
                List<Single> reversed = ReverseValues(values); // lista megfordítása
                Console.WriteLine("Adatok megfordítva:");
                Write(reversed);
            }

            Console.ReadKey();
        }
    }
}
